/*
** viper.c
**
** Get/set status/parameters for a CMD Viper RAID controller
**
** Copyright (c) 1998-2000 Peter Eriksson <pen@lysator.liu.se>
*/

#include <stdio.h>
#include <errno.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <math.h>
#include <signal.h>

#include <rts/rts.h>

#include "daemon.h"

extern char version[];

char *name = NULL;

int debug = 0;


char **devtab = NULL;
int devidx = 0;
int devtabsize = 0;

static double
f2c(double f)
{
    return (f - 32) / 1.8;
}


void
do_time(RTS *rp)
{
    int code;

    time_t c_t, o_t;

    code = rts_read_date_and_time(rp, &c_t);
    if (code)
    {
	perror("mktime");
	rts_perror(stderr, "rts_read_date_and_time", rp);
	exit(1);
    }
    
    time(&o_t);
    
    if (debug)
    {
	fprintf(stderr, "Our time:   %s\n", ctime(&o_t));
	fprintf(stderr, "Viper time: %s\n", ctime(&c_t));
    }
    
    if (fabs(difftime(o_t, c_t)) > 5)
    {
	
	code = rts_begin_session(rp, 0);
	if (code)
	{
	    rts_perror(stderr, "rts_begin_session", rp);
	    exit(1);
	}
	
	code = rts_write_date_and_time(rp, o_t);
	if (code)
	{
	    rts_perror(stderr, "rts_write_date_and_time", rp);
	    rts_end_session(rp);
	    exit(1);
	}

	
	code = rts_end_session(rp);
	if (code)
	{
	    rts_perror(stderr, "rts_end_session", rp);
	    exit(1);
	}
	
	syslog(LOG_NOTICE, "Time set on Viper");
    }
}


void
usage(FILE *fp)
{
    fprintf(fp, "Usage: get_temp [options] <device>\n");
    fprintf(fp, "Options:\n");
    fprintf(fp, "\t-V          Print version\n");
    fprintf(fp, "\t-d          Debug mode\n");
    fprintf(fp, "\t-h          Display this information\n");
}


int
main(int argc, char *argv[])
{
    RTS *rp;
    int c, code;
    time_t c_t;
    struct rts_vendor_parameters vpb;
    struct rts_environmental_information eib;
    struct rts_static_system_information ssib;
    struct rts_system_parameters spb;
    int print_vendor = 1;
    int print_env = 1;
    int print_static = 1;
    int print_system = 1;
    int print_misc = 1;
	
	
    while ((c = getopt(argc, argv, "Vdh")) != -1)
	switch (c)
	{
	  case 'V':
	    printf("Viperd %s, built on %s %s\n",
		   version, __DATE__, __TIME__);
	    printf("RTS Version: %d\n", RTS_VERSION);
	    exit(0);

	  case 'd':
	    debug = 1;
	    break;
	    
	  case 'h':
	    usage(stdout);
	    exit(0);
	    
	  default:
	    usage(stderr);
	    exit(1);
	}
    
    if (optind >= argc)
    {
	usage(stderr);
	exit(1);
    }
    
    rp = rts_open(argv[optind]);
    if (rp == NULL)
    {
	fprintf(stderr, "rts_open: %s", strerror(errno));
	exit(1);
    }

    if (print_static)
    {
	puts("\nStatic System Information:");
	
	code = rts_read_static_system_information(rp, &ssib);
	if (code)
	{
	    rts_perror(stderr, "rts_read_static_system_information", rp);
	    exit(1);
	}
	
	printf("  Firmware version:                  %.9s\n",
	       ssib.firmware_version);
	printf("  RTS version:                       %d\n",
	       ssib.rts_version);
	printf("  Model number:                      %d\n",
	       ssib.model_number);
	printf("  Processor type:                    %d\n",
	       ssib.processor_type);
	printf("  Processor clock:                   %d MHz\n",
	       ssib.processor_clock);
	printf("  Processor memory:                  %d MB\n",
	       ssib.processor_memory);
	printf("  Cache size:                        %d MB\n",
	       rts_w2i(ssib.cache_size));
	printf("  SIMM 0A size:                      %d MB\n",
	       ssib.simm_0a_size);
	printf("  SIMM 0B size:                      %d MB\n",
	       ssib.simm_0b_size);
	printf("  SIMM 1A size:                      %d MB\n",
	       ssib.simm_1a_size);
	printf("  SIMM 1B size:                      %d MB\n",
	       ssib.simm_1b_size);
	printf("  Serial number:                     %.9s\n",
	       ssib.serial_number);
	printf("  Manufacture date:                  %d/%d/%d\n",
	       ssib.manufacture_month,
	       ssib.manufacture_date,
	       ssib.manufacture_year);
	
	/* XXX: Channel status / DMA vendor ID ? */
	
	printf("  DRAM 0 vendor ID:                  %d\n",
	       ssib.dram_0_vendor_id);
	printf("  DRAM 1 vendor ID:                  %d\n",
	       ssib.dram_1_vendor_id);
	printf("  XOR vendor ID:                     %d\n",
	       ssib.xor_vendor_id);
	printf("  PIC vendor ID:                     %d\n",
	       ssib.pic_vendor_id);
    }

    if (print_vendor)
    {
	puts("\nVendor Parameters:");
	
	code = rts_read_vendor_parameters(rp, &vpb);
	if (code != 0)
	{
	    rts_perror(stderr, "rts_read_vendor_parameters failed", rp);
	    exit(1);
	}
	
	printf("  Display Vendor:                    %.16s\n",
	       vpb.system_display_vendor);
	
	printf("  Display Model:                     %.16s\n",
	       vpb.system_display_model);
	
	printf("  Inquiry Vendor:                    %.8s\n",
	       vpb.inquiry_response_vendor);
	
	printf("  Inquiry Model:                     %.16s\n",
	       vpb.inquiry_response_model);
	
	
#if RTS_VERSION >= 2
	printf("  Write back cache:                  %d %%\n",
	       vpb.percent_write_back_cache);
	
	printf("  Enclose severe temperature level:  %d\n",
	       vpb.enclosure_severe_level_temp);
#endif
	
#if RTS_VERSION >= 5
	printf("  Enclose warning temperature level: %d\n",
	       vpb.enclosure_warning_level_temp);
#endif
#if RTS_VERSION >= 7
	printf("  Extended acknowledge mode:         %d\n",
	       vpb.extended_acknowledge_mode);
#endif
    }

    if (print_env)
    {
	puts("\nEnvironmental Parameters:");
	
	code = rts_read_environmental_information(rp, &eib);
	if (code != 0)
	{
	    rts_perror(stderr, "rts_read_environmental_parameters", rp);
	    exit(1);
	}

	printf("  System board +5V level:            %.2f V\n",
	       ((double) rts_w2i(eib.five_volt_level)) / 100.0);
	
	printf("  System board +12V level:           %.2f V\n",
	       ((double) rts_w2i(eib.twelve_volt_level)) / 100.0);
	
	printf("  SCSI Termination +5V level:        %.2f V\n",
	       ((double) rts_w2i(eib.scsi_termination_volt_level)) / 100.0);
	
	printf("  DRAM bank 0 +5V level:             %.2f V\n",
	       ((double) rts_w2i(eib.dram_0_volt_level)) / 100.0);
	
	printf("  DRAM bank 1 +5V level:             %.2f V\n",
	       ((double) rts_w2i(eib.dram_1_volt_level)) / 100.0);
	
	printf("  Backup battery volt level:         %.2f V\n",
	       ((double) rts_w2i(eib.backup_battery_volt_level)) / 100.0);
	
	printf("  Board temperature level:           %d F (%d C)\n",
	       eib.board_temperature_level,
	       (int) (f2c(eib.board_temperature_level)+0.5));
	
	printf("  Enclosure temperature level:       %d F (%d C)\n",
	       eib.enclosure_temperature_level,
	       (int) (f2c(eib.enclosure_temperature_level)+0.5));
    }


    if (print_system)
    {
	puts("\nSystem Parameters:");
	
	code = rts_read_system_parameters(rp, &spb);
	if (code)
	{
	    rts_perror(stderr, "rts_read_system_parameters", rp);
	    exit(1);
	}
	
	printf("  Password checking:                 %d\n",
	       spb.password_checking);
	printf("  Rebuild rate:                      %d\n",
	       spb.rebuild_rate);
	printf("  Create rate:                       %d\n",
	       spb.create_rate);
	printf("  UPS installed:                     %d\n",
	       spb.ups_installed);
	printf("  Channel 1 module type:             %d\n",
	       spb.ch1_module_type);
	printf("  Channel 2 module type:             %d\n",
	       spb.ch2_module_type);
	printf("  Channel 3 module type:             %d\n",
	       spb.ch3_module_type);
	printf("  Read ahead enabled:                %d\n",
	       spb.read_ahead_enabled);
	printf("  Read ahead limit:                  %d\n",
	       spb.read_ahead_limit);
	printf("  Validation delay:                  %d\n",
	       spb.validation_delay);
	printf("  Twelve volt sensor enabled:        %d\n",
	       spb.twelve_volt_sensor_enabled);
	printf("  Host WCE enabled:                  %d\n",
	       spb.host_wce_enabled);
	printf("  Drive command timeout:             %d\n",
	       spb.drive_command_timeout);
	printf("  Reset propagation:                 %d\n",
	       spb.reset_propagation);
	printf("  Environmental alerts:              %d\n",
	       spb.environmental_alerts);
	printf("  Spinup staggering:                 %d\n",
	       spb.spinup_staggering);
    }

    if (print_misc)
    {
	puts("\nMiscellaneous:");
	
	code = rts_read_date_and_time(rp, &c_t);
	if (code)
	{
	    rts_perror(stderr, "rts_read_date_and_time", rp);
	    exit(1);
	}
	
	printf("  Clock on controller:               %s\n", ctime(&c_t));
    }
	   
    exit(0);
}
